<?php

namespace Drupal\role_hierarchy\Service;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\user\Entity\Role;
use Drupal\user\Entity\User;
use Drupal\user\UserInterface;

/**
 * Class RoleHierarchyHelper provides core functions for role_hierarchy.
 *
 * @package Drupal\role_hierarchy
 */
class RoleHierarchyHelper {

  /**
   * @var \Drupal\Core\Config\ImmutableConfig
   *
   * The module settings.
   */
  protected $settings;

  /**
   * RoleHierarchyHelper constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config factory.
   */
  public function __construct(ConfigFactoryInterface $configFactory) {
    $this->settings = $configFactory->get('role_hierarchy.settings');
  }

  /**
   * Get the lowest/highest user role, depending on settings.
   *
   * @param \Drupal\user\UserInterface $user
   *   The current user.
   *
   * @return int
   *   The user  role weight.
   */
  public function getUserRoleWeight(UserInterface $user) {
    return $this->getAccountRoleWeight($user);
  }

  /**
   * Get the lowest/highest account role, depending on settings.
   *
   * @param \Drupal\Core\Session\AccountInterface $account
   *   The current account.
   *
   * @return int
   *   The account role weight.
   */
  public function getAccountRoleWeight(AccountInterface $account) {
    $highest_role = $this->getAccountHighestRole($account);

    if (empty($highest_role)) {
      return !empty($this->settings->get('invert'))
        ? -9999
        : 9999;
    }

    return $highest_role->getWeight();
  }

  /**
   * Get the lowest/highest account role, depending on settings.
   *
   * @param \Drupal\Core\Session\AccountInterface $account
   *   The current account.
   *
   * @return \Drupal\user\RoleInterface
   *   The account role.
   */
  public function getAccountHighestRole(AccountInterface $account) {
    $account_roles = Role::loadMultiple(array_values($account->getRoles()));
    $non_hierarchical_roles = $this->settings->get('non_hierarchical_roles') ?? [];
    $account_roles = array_filter($account_roles, function ($role) use ($non_hierarchical_roles) {
      return empty($non_hierarchical_roles[$role->id()]);
    });

    if (empty($account_roles)) {
      return NULL;
    }

    $sorted_account_roles = [];
    foreach ($account_roles as $account_role) {
      $sorted_account_roles[$account_role->id()] = $this->getRoleWeight($account_role);
    }
    asort($sorted_account_roles);

    if (!empty($this->settings->get('invert'))) {
      return Role::load(array_key_last($account_roles));
    }

    return Role::load(array_key_first($account_roles));
  }

  /**
   * Get the weight of a role as configured in /admin/people/roles.
   *
   * @param Role|string $role
   *   The user role.
   *
   * @return int
   *   The weight of the role.
   */
  public function getRoleWeight($role) {
    if ($role == 'workflow_author') {
      return 9999;
    }

    if (is_string($role)) {
      $role = Role::load($role);
    }

    return $role->getWeight();
  }

  /**
   * Check if an user can edit an user role by comparing their weights.
   *
   * @param \Drupal\Core\Session\AccountInterface
   *   The user editing the role.
   * @param $edited_role
   *   The edited user role.
   *
   * @return bool
   */
  public function hasRoleEditAccess(AccountInterface $account, $edited_role) {
    if ($account->hasPermission('bypass role hierarchy')) {
      return TRUE;
    }

    $user_role_weight = $this->getAccountRoleWeight($account);
    $edited_role_weight = $this->getRoleWeight($edited_role);

    if (!empty($this->settings->get('invert'))) {
      if (!empty($this->settings->get('strict'))) {
        return $user_role_weight > $edited_role_weight;
      }
      else {
        return $user_role_weight >= $edited_role_weight;
      }
    }
    else {
      if (!empty($this->settings->get('strict'))) {
        return $user_role_weight < $edited_role_weight;
      }
      else {
        return $user_role_weight <= $edited_role_weight;
      }
    }
  }

  /**
   * Determines if an account has edit access on an user.
   *
   * @param AccountInterface $account
   *   The account that tries to edit.
   *
   * @param User $user
   *   The user.
   *
   * @return bool
   */
  public function hasEditAccess(AccountInterface $account, User $user) {
    if ($account->id() == 1) {
      return TRUE;
    }

    if ($account->hasPermission('bypass role hierarchy')) {
      return TRUE;
    }

    if ($account->id() == $user->id()) {
      return TRUE;
    }

    if ($user->id() == 1) {
      return FALSE;
    }

    return $this->hasRoleEditAccess($account, $this->getAccountHighestRole($user));
  }

}
